@@ -155,13 +155,33 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
         assert node_id is not None
         assert prop_name is not None
         assert prop_val is not None
-        # note that we are not copying node properties, which means the next line modifies
+
         if prop_name == self.NETWORKX_LABEL:
             raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
                                               msg=f"Changing {self.NETWORKX_LABEL} property is not permitted")
+        # note that we are not copying node properties, which means the next line modifies
         node_props = self.storage.get_graph(self.graph_id).nodes[self._find_node(node_id=node_id)]
         node_props[prop_name] = prop_val
 
+    def unset_node_property(self, *, node_id: str, prop_name: str) -> None:
+        assert node_id is not None
+        assert prop_name is not None
+
+        if prop_name == self.NETWORKX_LABEL:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
+                                              msg=f"Unsetting {self.NETWORKX_LABEL} property is not permitted")
+        if prop_name in ABCPropertyGraph.NO_UNSET_PROPERTIES:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
+                                              msg=f"Unsetting property {prop_name} is not allowed)")
+
+        # since this is not a copy, after that we modify live graph
+        node_props = self.storage.get_graph(self.graph_id).nodes[self._find_node(node_id=node_id)]
+        if prop_name in node_props.keys():
+            node_props.pop(prop_name)
+        else:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_id,
+                                              msg=f"Unable to unset property {prop_name}")
+
     def update_nodes_property(self, *, prop_name: str, prop_val: Any) -> None:
         """
         update a selected property on all nodes of the graph
@@ -229,6 +249,31 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
                                               msg="Link of this type doesn't exist")
         edge_props[prop_name] = prop_val
 
+    def unset_link_property(self, *, node_a: str, node_b: str, kind: str, prop_name: str) -> None:
+        assert node_a is not None
+        assert node_b is not None
+        assert kind is not None
+        assert prop_name is not None
+
+        if prop_name == self.NETWORKX_LABEL:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=None,
+                                              msg=f"Unsetting {self.NETWORKX_LABEL} property is not permitted")
+        # find nodes matching those IDs and get link properties
+        real_node_a = self._find_node(node_id=node_a)
+        real_node_b = self._find_node(node_id=node_b)
+        try:
+            # not a copy
+            edge_props = self.storage.get_graph(self.graph_id).edges[(real_node_a, real_node_b)]
+        except KeyError:
+            # no edge exists
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_a, msg="Link doesn't exist")
+        etype = edge_props.get(self.NETWORKX_LABEL, None)
+        if etype != kind:
+            raise PropertyGraphQueryException(graph_id=self.graph_id, node_id=node_a,
+                                              msg="Link of this type doesn't exist")
+        if prop_name in edge_props.keys():
+            edge_props.pop(prop_name)
+
     def update_link_properties(self, *, node_a: str, node_b: str, kind: str, props: Dict[str, Any]) -> None:
         """
         update multiple properties of a link overriding existing ones if necessary.
@@ -261,6 +306,21 @@ class NetworkXPropertyGraph(ABCPropertyGraph, NetworkXMixin):
                                               msg="Link of this type doesn't exist")
         edge_props.update(props)
 
+    def get_all_nodes_by_class(self, *, label: str) -> List[str]:
+        assert label is not None
+        my_graph = self.storage.get_graph(self.graph_id)
+        graph_nodes = list(nxq.search_nodes(my_graph,
+                                            {'and': [
+                                                {'eq': [ABCPropertyGraph.GRAPH_ID, self.graph_id]},
+                                                {'eq': [ABCPropertyGraph.PROP_CLASS,
+                                                        label]}
+                                            ]
+                                            }))
+        ret = list()
+        for n in graph_nodes:
+            ret.append(my_graph.nodes[n][ABCPropertyGraph.NODE_ID])
+        return ret
+
     def list_all_node_ids(self) -> List[str]:
         """
         List all NodeID properties of nodes in a graph
@@ -749,3 +809,10 @@ class NetworkXGraphImporter(ABCGraphImporter):
         :return:
         """
         self.storage.del_all_graphs()
+
+    def cast_graph(self, *, graph_id: str) -> ABCPropertyGraph:
+
+        assert graph_id is not None
+        neo4jg = NetworkXPropertyGraph(graph_id=graph_id, importer=self, logger=self.log)
+        assert neo4jg.graph_exists()
+        return neo4jg
